<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 08/31/2003 *
 * ===================                          Last change: 11/23/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : admin-inc.php                                    *
 * -------------------------------------------------------------------- *
 * Short description : Administrative related functions                 *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Fuer die Administration benoetigte Funktionen    *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Register an administrator account
function addAdminAccount ($adminLogin, $passHash, $adminEmail, $accessLevel = 'deny') {
	// Login does already exist
	$ret = 'already';

	// Lookup the admin
	$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_admins` WHERE `login`='%s' LIMIT 1",
		array($adminLogin), __FUNCTION__, __LINE__);

	// Is the entry there?
	if (SQL_HASZERONUMS($result)) {
		// Is ext-admins installed and version at least 0.3.0?
		if (isExtensionInstalledAndNewer('admins', '0.3.0')) {
			// Ok, let's create the admin login
			SQL_QUERY_ESC("INSERT INTO `{?_MYSQL_PREFIX?}_admins` (`login`, `password`, `email`, `default_acl`) VALUES ('%s', '%s', '%s', '%s')",
				array(
					$adminLogin,
					$passHash,
					$adminEmail,
					$accessLevel
				), __FUNCTION__, __LINE__);
		} else {
			// Ok, let's create the admin login
			SQL_QUERY_ESC("INSERT INTO `{?_MYSQL_PREFIX?}_admins` (`login`, `password`, `email`) VALUES ('%s', '%s', '%s')",
				array(
					$adminLogin,
					$passHash,
					$adminEmail
				), __FUNCTION__, __LINE__);
		}

		// All done
		$ret = 'done';
	} // END - if

	// Free memory
	SQL_FREERESULT($result);

	// Return result
	return $ret;
}

// This function will be executed when the admin is not logged in and has submitted his login data
function ifAdminLoginDataIsValid ($adminLogin, $adminPassword) {
	// First of all, no admin login is found, so the admin hash is null
	$ret = '404';
	$adminHash = NULL;

	// Get admin id from login
	$adminId = getAdminId($adminLogin);

	// Continue only with found admin ids
	if (isValidId($adminId)) {
		// Then we need to lookup the login name by getting the admin hash
		$adminHash = getAdminHash($adminId);

		// If this is fine, we can continue
		if ($adminHash != '-1') {
			// Get admin id and set it as current
			setCurrentAdminId($adminId);

			// Now, we need to encode the password in the same way the one is encoded in database
			$testHash = generateHash($adminPassword, $adminHash);

			// If they both match, the login data is valid
			if ($testHash == $adminHash) {
				// All fine
				$ret = 'done';
			} else {
				// Did not match!
				$ret = 'password';
			}
		} // END - if
	} // END - if

	// Prepare data array
	$data = array(
		'id'         => $adminId,
		'login'      => $adminLogin,
		'plain_pass' => $adminPassword,
		'pass_hash'  => $adminHash
	);

	// Run a special filter
	runFilterChain('do_admin_login_' . $ret, $data);

	// Return status
	return $ret;
}

// Only be executed on cookie checking
function ifAdminCookiesAreValid ($adminLogin, $passHash) {
	// First of all, no admin login is found
	$ret = '404';

	// Then we need to lookup the login name by getting the admin hash
	$adminHash = getAdminHash($adminLogin);

	// If this is fine, we can continue
	if ($adminHash != '-1') {
		// Now, we need to encode the password in the same way the one is encoded in database
		$testHash = encodeHashForCookie($adminHash);
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'adminLogin=' . $adminLogin . ',passHash='.$passHash.',adminHash='.$adminHash.',testHash='.$testHash);

		// If they both match, the login data is valid
		if ($testHash != $passHash) {
			// Passwords don't match
			$ret = 'password';
		} elseif (!isAdmin()) {
			// Is not valid session
			$ret = 'session';
		} else {
			// All fine
			$ret = 'done';
		}
	} // END - if

	// Return status
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'ret='.$ret);
	return $ret;
}

// Do an admin action
function doAdminAction () {
	// Determine correct 'what' value
	$what = determineWhat();

	// Get action value
	$action = getActionFromModuleWhat(getModule(), $what);

	// Load welcome template
	if (isExtensionActive('admins')) {
		// @TODO This and the next getCurrentAdminId() call might be moved into the templates?
		$content['welcome'] = loadTemplate('admin_welcome_admins', TRUE, getCurrentAdminId());
	} else {
		$content['welcome'] = loadTemplate('admin_welcome', TRUE, getCurrentAdminId());
	}

	// Load header, footer, render menu
	$content['header'] = loadTemplate('admin_header' , TRUE, $content);
	$content['footer'] = loadTemplate('admin_footer' , TRUE, $content);
	$content['menu']   = addAdminMenu($action, $what);

	// Load main template
	loadTemplate('admin_main', FALSE, $content);

	// Check if action/what pair is valid
	$result_action = SQL_QUERY_ESC("SELECT
	`id`
FROM
	`{?_MYSQL_PREFIX?}_admin_menu`
WHERE
	`action`='%s' AND
	(
		(
			`what`='%s' AND `what` != 'welcome'
		) OR (
			(
				`what`='' OR `what` IS NULL
			) AND (
				'%s'='welcome'
			)
		)
	)
LIMIT 1",
		array(
			$action,
			$what,
			$what
		), __FUNCTION__, __LINE__);

	// Is there an entry?
	if (SQL_NUMROWS($result_action) == 1) {
		// Is valid but does the inlcude file exists?
		$inc = sprintf("inc/modules/admin/action-%s.php", $action);
		if ((isIncludeReadable($inc)) && (isMenuActionValid('admin', $action, $what)) && ($GLOBALS['acl_allow'] === TRUE)) {
			// Ok, we finally load the admin action module
			loadInclude($inc);
		} elseif ($GLOBALS['acl_allow'] === FALSE) {
			// Access denied
			loadTemplate('admin_menu_failed', FALSE, '{%message,ADMIN_ACCESS_DENIED=' . $what . '%}');
		} else {
			// Include file not found :-(
			loadTemplate('admin_menu_failed', FALSE, '{%message,ADMIN_ACTION_404=' . $action . '%}');
		}
	} else {
		// Invalid action/what pair found
		loadTemplate('admin_menu_failed', FALSE, '{%message,ADMIN_ACTION_INVALID=' . $action . '/' . $what . '%}');
	}

	// Free memory
	SQL_FREERESULT($result_action);

	// Tableset footer
	loadTemplate('admin_main_footer', FALSE, $content);
}

/**
 * Checks whether current admin is allowed to access given action/what
 * combination (only one is allowed to be null!).
 */
function isAdminAllowedAccessMenu ($action, $what = NULL) {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__][$action][$what])) {
		// ACL is always 'allow' when no ext-admins is installed
		// @TODO This can be rewritten into a filter
		$GLOBALS[__FUNCTION__][$action][$what] = ((!isExtensionInstalledAndNewer('admins', '0.2.0')) || ((isExtensionActive('admins')) && (isAdminsAllowedByAcl($action, $what))));
	} // END - if

	// Return the cached value
	return $GLOBALS[__FUNCTION__][$action][$what];
}

// Adds an admin menu
function addAdminMenu ($action, $what) {
	// Init variables
	$SUB = FALSE;
	$OUT = '';

	// Menu descriptions
	$GLOBALS['menu']['description'] = array();
	$GLOBALS['menu']['title']       = array();

	// Build main menu
	$result_main = SQL_QUERY("SELECT
	`action` AS `main_action`,
	`title` AS `main_title`,
	`descr` AS `main_descr`
FROM
	`{?_MYSQL_PREFIX?}_admin_menu`
WHERE
	(`what`='' OR `what` IS NULL)
ORDER BY
	`sort` ASC,
	`id` DESC", __FUNCTION__, __LINE__);

	// Are there entries?
	if (!SQL_HASZERONUMS($result_main)) {
		$OUT .= '<ul class="admin_menu_main">';

		// Load all 'action' menus
		while ($mainContent = SQL_FETCHARRAY($result_main)) {
			// Filename
			$inc = sprintf("inc/modules/admin/action-%s.php", $mainContent['main_action']);

			// Is the current admin allowed to access this 'action' menu?
			if (isAdminAllowedAccessMenu($mainContent['main_action'])) {
				if ($SUB === FALSE) {
					// Insert compiled menu title and description
					$GLOBALS['menu']['title'][$mainContent['main_action']]       = $mainContent['main_title'];
					$GLOBALS['menu']['description'][$mainContent['main_action']] = $mainContent['main_descr'];
				} // END - if
				$OUT .= '<li class="admin_menu"' . addJavaScriptMenuContent('admin', $mainContent['main_action'], $action, $what) . '>
<div class="nobr"><strong>&middot;</strong>&nbsp;';

				// Is the file readable?
				if (isIncludeReadable($inc)) {
					if (($mainContent['main_action'] == $action) && (empty($what))) {
						$OUT .= '<strong>';
					} else {
						$OUT .= '[<a href="{%url=modules.php?module=admin&amp;action=' . $mainContent['main_action'] . '%}">';
					}
				} else {
					$OUT .= '<span class="bad" style="cursor:help" title="{%message,ADMIN_MENU_ACTION_404_TITLE=' . $mainContent['main_action'] . '%}">';
				}

				$OUT .= $mainContent['main_title'];

				// Is the file readable?
				if (isIncludeReadable($inc)) {
					if (($mainContent['main_action'] == $action) && (empty($what))) {
						$OUT .= '</strong>';
					} else {
						$OUT .= '</a>]';
					}
				} else {
					$OUT .= '</span>';
				}

				$OUT .= '</div>
</li>';

				// Add sub menu
				$OUT .= addAdminSubMenu($mainContent, $action, $what);
			} // END - if
		} // END - while

		// Close ul-tag
		$OUT .= '</ul>';

		// Free memory
		SQL_FREERESULT($result_main);
	} // END - if

	// Return content
	return $OUT;
}

// Add admin sub menu
function addAdminSubMenu ($mainContent, $action, $what) {
	// Init content
	$OUT = '';

	// Check for menu entries
	$result_what = SQL_QUERY_ESC("SELECT
	`what` AS `sub_what`,
	`title` AS `sub_title`,
	`descr` AS `sub_descr`
FROM
	`{?_MYSQL_PREFIX?}_admin_menu`
WHERE
	`action`='%s' AND
	`what` != '' AND
	`what` IS NOT NULL
ORDER BY
	`sort` ASC,
	`id` DESC",
		array($mainContent['main_action']), __FUNCTION__, __LINE__);

	// Remember the count for later checks
	setAdminMenuHasEntries($mainContent['main_action'], ((!SQL_HASZERONUMS($result_what)) && (($action == $mainContent['main_action']) || (isAdminMenuJavascriptEnabled()))));

	// Start li-tag for sub menu content
	$OUT .= '<li class="admin_menu_sub" id="action_menu_' . $mainContent['main_action'] . '"' . addStyleMenuContent('admin', $mainContent['main_action'], $action) . '>';

	// Are there entries?
	if (ifAdminMenuHasEntries($mainContent['main_action'])) {
		// Sub menu has been called
		$SUB = TRUE;

		// Are there entries?
		if (!SQL_HASZERONUMS($result_what)) {
			// Start HTML code
			$OUT .= '<ul class="admin_menu_sub">';

			// Load all entries
			while ($subContent = SQL_FETCHARRAY($result_what)) {
				// Filename
				$inc = sprintf("inc/modules/admin/what-%s.php", $subContent['sub_what']);

				// Is the current admin allowed to access this 'what' menu?
				if (isAdminAllowedAccessMenu(NULL, $subContent['sub_what'])) {
					// Insert compiled title and description
					$GLOBALS['menu']['title'][$subContent['sub_what']]       = $subContent['sub_title'];
					$GLOBALS['menu']['description'][$subContent['sub_what']] = $subContent['sub_descr'];
					$OUT .= '<li class="admin_menu">
<div class="nobr"><strong>--&gt;</strong>&nbsp;';

					// Is the file readable?
					if (isIncludeReadable($inc)) {
						if ($what == $subContent['sub_what']) {
							$OUT .= '<strong>';
						} else {
							$OUT .= '[<a href="{%url=modules.php?module=admin&amp;what=' . $subContent['sub_what'] . '%}">';
						}
					} else {
						$OUT .= '<span class="bad" style="cursor:help" title="{%message,ADMIN_MENU_WHAT_404_TITLE=' . $subContent['sub_what'] . '%}">';
					}

					$OUT .= $subContent['sub_title'];

					// Is the file readable?
					if (isIncludeReadable($inc)) {
						if ($what == $subContent['sub_what']) {
							$OUT .= '</strong>';
						} else {
							$OUT .= '</a>]';
						}
					} else {
						$OUT .= '</span>';
					}
					$OUT .= '</div>
</li>';
				} // END - if
			} // END - while

			// Finish HTML output
			$OUT .= '</ul>';
		} // END - if

		// Free memory
		SQL_FREERESULT($result_what);
	} // END - if

	// Close li-tag
	$OUT .= '</li>';

	// Return content
	return $OUT;
}

// Create an admin selection box form
function addAdminSelectionBox ($adminId = NULL, $special = '') {
	// Default is email as "special column"
	$ADD = ',`email` AS `special`';

	// Is a special column given?
	if (!empty($special)) {
		// Additional column for SQL query
		$ADD = ',`' . $special . '` AS `special`';
	} // END - if

	// Query all entries
	$result = SQL_QUERY('SELECT
	`id`,
	`login`
	' . $ADD . '
FROM
	`{?_MYSQL_PREFIX?}_admins`
ORDER BY
	`login` ASC', __FUNCTION__, __LINE__);

	// Init output
	$OUT = '';

	// Load all entries
	while ($content = SQL_FETCHARRAY($result)) {
		// Default is none
		$content['default'] = '';

		// Is the id the same?
		if ($content['id'] == $adminId) {
			// Set this as default
			$content['default'] = ' selected="selected"';
		} // END - if

		// Add the entry
		$OUT .= loadTemplate('select_admins_option', TRUE, $content);
	} // END - if

	// Free memory
	SQL_FREERESULT($result);

	// Add form to content
	$content['form_selection'] = $OUT;

	// Output form
	loadTemplate('select_admins_box', FALSE, $content);
}

// Create a member selection box
function addMemberSelectionBox ($userid = NULL, $add_all = FALSE, $return = FALSE, $none = FALSE, $field = 'userid', $whereStatement = " WHERE `surname` NOT LIKE '{?tester_user_surname_prefix?}%'") {
	// Output selection form with all confirmed user accounts listed
	$result = SQL_QUERY('SELECT
	`userid`,
	`surname`,
	`family`
FROM
	`{?_MYSQL_PREFIX?}_user_data`
' . $whereStatement . '
ORDER BY
	`userid` ASC', __FUNCTION__, __LINE__);

	// Default output
	$OUT = '';

	// USe this only for adding points (e.g. adding refs really makes no sence ;-) )
	if ($add_all === TRUE) {
		$OUT = '      <option value="all">{--ALL_MEMBERS--}</option>';
	} elseif ($none === TRUE) {
		$OUT = '      <option value="0">{--SELECT_NONE--}</option>';
	}

	// Load all entries
	while ($content = SQL_FETCHARRAY($result)) {
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . intval($userid) . '/' . $content['userid']);
		$OUT .= '<option value="' . bigintval($content['userid']) . '"';
		if (bigintval($userid, FALSE, FALSE) === bigintval($content['userid'])) {
			$OUT .= ' selected="selected"';
		} // END - if
		$OUT .= '>' . $content['surname'] . ' ' . $content['family'] . ' (' . bigintval($content['userid']) . ')</option>';
	} // END - while

	// Free memory
	SQL_FREERESULT($result);

	if ($return === FALSE) {
		// Remeber options in constant
		$content['form_selection'] = $OUT;
		$content['what']           = '{%pipe,getWhat%}';

		// Load template
		loadTemplate('admin_form_selection_box', FALSE, $content);
	} else {
		// Return content in selection frame
		return '<select class="form_select" name="' . handleFieldWithBraces($field) . '" size="1">' . $OUT . '</select>';
	}
}

// Create a menu selection box for given menu system
// @TODO Try to rewrite this to adminAddMenuSelectionBox()
// @DEPRECATED
function adminMenuSelectionBox_DEPRECATED ($mode, $default = '', $defid = '') {
	$what = "`what` != '' AND `what` IS NOT NULL";
	if ($mode == 'action') $what = "(`what`='' OR `what` IS NULL) AND `action` != 'login'";

	$result = SQL_QUERY_ESC("SELECT `%s` AS `menu`, `title` FROM `{?_MYSQL_PREFIX?}_admin_menu` WHERE ".$what." ORDER BY `sort` ASC",
		array($mode), __FUNCTION__, __LINE__);
	if (!SQL_HASZERONUMS($result)) {
		// Load menu as selection
		$OUT = '<select name="' . $mode . '_menu';
		if (!isValidId($defid)) $OUT .= '[' . intval($defid) . ']';
		$OUT .= '" size="1" class="form_select">
	<option value="">{--SELECT_NONE--}</option>';
		// Load all entries
		while ($content = SQL_FETCHARRAY($result)) {
			$OUT .= '<option value="' . $content['menu'] . '"';
			if ((!empty($default)) && ($default == $content['menu'])) {
				$OUT .= ' selected="selected"';
			} // END - if
			$OUT .= '>' . $content['title'] . '</option>';
		} // END - while

		// Free memory
		SQL_FREERESULT($result);

		// Add closing select-tag
		$OUT .= '</select>';
	} else {
		// No menus???
		$OUT = '{--ADMIN_PROBLEM_NO_MENU--}';
	}

	// Return output
	return $OUT;
}

// Wrapper for $_POST and adminSaveSettings
function adminSaveSettingsFromPostData ($tableName = '_config', $whereStatement = '`config`=0', $translateComma = array(), $alwaysAdd = FALSE, $displayMessage = TRUE) {
	// Get the array
	$postData = postRequestArray();

	// Call the lower function
	adminSaveSettings($postData, $tableName, $whereStatement, $translateComma, $alwaysAdd, $displayMessage);
}

// Save settings to the database
function adminSaveSettings (&$postData, $tableName = '_config', $whereStatement = '`config`=0', $translateComma = array(), $alwaysAdd = FALSE, $displayMessage = TRUE) {
	// Prepare all arrays, variables
	$tableData = array();
	$skip = FALSE;

	// Now, walk through all entries and prepare them for saving
	//* BUG: */ reportBug(__FUNCTION__, __LINE__, '<pre>'.print_r(postRequestArray(), TRUE).'</pre>');
	foreach ($postData as $id => $val) {
		// Process only formular field but not submit buttons ;)
		if ($id == 'save_config') {
			// Skip this button
			continue;
		} // END - if

		// Do not save the ok value
		convertSelectionsToEpocheTime($postData, $tableData, $id, $skip);

		// Shall we process this id? It muss not be empty, of course
		if (($skip === FALSE) && (!empty($id)) && ((!isset($GLOBALS['skip_config'][$id]))) || ($tableName != '_config')) {
			// Translate the value? (comma to dot!)
			if ((is_array($translateComma)) && (in_array($id, $translateComma))) {
				// Then do it here... :)
				$val = convertCommaToDot($val);
			} // END - if

			// Test value on float
			$test = (float) $val;

			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'test=' . $test . ',val=' . $val . ',id=' . $id);

			// Shall we add numbers or strings?
			if ('' . $val . '' == '' . $test . '') {
				// Add numbers
				array_push($tableData, sprintf("`%s`=%s", $id, $test));
			} elseif (is_null($val)) {
				// Add NULL
				array_push($tableData, sprintf("`%s`=NULL", $id));
			} else {
				// Add strings
				array_push($tableData, sprintf("`%s`='%s'", $id, trim($val)));
			}

			// Do not add a config entry twice
			$GLOBALS['skip_config'][$id] = TRUE;

			// Update current configuration
			setConfigEntry($id, $val);
		} // END - if
	} // END - foreach

	// Check if entry does exist
	$result = FALSE;
	if ($alwaysAdd === FALSE) {
		if (!empty($whereStatement)) {
			$result = SQL_QUERY("SELECT * FROM `{?_MYSQL_PREFIX?}" . $tableName . "` WHERE " . $whereStatement . " LIMIT 1", __FUNCTION__, __LINE__);
		} else {
			$result = SQL_QUERY("SELECT * FROM `{?_MYSQL_PREFIX?}" . $tableName . "` LIMIT 1", __FUNCTION__, __LINE__);
		}
	} // END - if

	if (SQL_NUMROWS($result) == 1) {
		// "Implode" all data to single string
		$updatedData = implode(', ', $tableData);

		// Generate SQL string
		$sql = sprintf("UPDATE `{?_MYSQL_PREFIX?}%s` SET %s WHERE %s LIMIT 1",
			$tableName,
			$updatedData,
			$whereStatement
		);
	} else {
		// Add Line (does only work with AUTO_INCREMENT!
		$keys = array(); $values = array();
		foreach ($tableData as $entry) {
			// Split up
			$line = explode('=', $entry);
			array_push($keys  , $line[0]);
			array_push($values, $line[1]);
		} // END - foreach

		// Add both in one line
		$keys   = implode('`, `', $keys);
		$values = implode(', '  , $values);

		// Generate SQL string
		$sql = sprintf("INSERT INTO `{?_MYSQL_PREFIX?}%s` (%s) VALUES (%s)",
			$tableName,
			$keys,
			$values
		);
	}

	// Free memory
	SQL_FREERESULT($result);

	// Simply run generated SQL string
	SQL_QUERY($sql, __FUNCTION__, __LINE__);

	// Remember affected rows
	$affected = SQL_AFFECTEDROWS();

	// Rebuild cache
	rebuildCache('config', 'config');

	// Settings saved, so display message?
	if ($displayMessage === TRUE) {
		// Display a message
		displayMessage('{--SETTINGS_SAVED--}');
	} // END - if

	// Return affected rows
	return $affected;
}

// Generate a selection box
function adminAddMenuSelectionBox ($menu, $type, $name, $default = '') {
	// Open the requested menu directory
	$menuArray = getArrayFromDirectory(sprintf("inc/modules/%s/", $menu), $type . '-', FALSE, FALSE);

	// Init the selection box
	$OUT = '<select name="' . $name . '" class="form_select" size="1"><option value="">{--ADMIN_IS_TOP_MENU--}</option>';

	// Walk through all files
	foreach ($menuArray as $file) {
		// Is this a PHP script?
		if ((!isDirectory($file)) && (isInString('' . $type . '-', $file)) && (isInString('.php', $file))) {
			// Then test if the file is readable
			$test = sprintf("inc/modules/%s/%s", $menu, $file);

			// Is the file there?
			if (isIncludeReadable($test)) {
				// Extract the value for what=xxx
				$part = substr($file, (strlen($type) + 1));
				$part = substr($part, 0, -4);

				// Is that part different from the overview?
				if ($part != 'welcome') {
					$OUT .= '<option value="' . $part . '"';
					if ($part == $default) {
						$OUT .= ' selected="selected"';
					} // END - if
					$OUT .= '>' . $part . '</option>';
				} // END - if
			} // END - if
		} // END - if
	} // END - foreach

	// Close selection box
	$OUT .= '</select>';

	// Return contents
	return $OUT;
}

// Creates a user-profile link for the admin. This function can also be used for many other purposes
function generateUserProfileLink ($userid, $title = '', $what = '') {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__][$userid][$title . '_' . $what])) {
		// Is title empty and valid userid?
		if (($title == '') && (isValidId($userid))) {
			// Set userid as title
			$title = $userid;
		} elseif (!isValidId($userid)) {
			// User id zero is invalid
			return '<strong>' . convertNullToZero($userid) . '</strong>';
		}

		// Is what set?
		if (empty($what)) {
			// Then get it to 'list_user'
			$what = 'list_user';
		} // END - if

		if (($title == '0') && ($what == 'list_refs')) {
			// Return title again
			return $title;
		} elseif (!empty($title)) {
			// Not empty, so skip next one
		} elseif (isExtensionActive('nickname')) {
			// Get nickname
			$nick = getNickname($userid);

			// Is it not empty, use it as title else the userid
			if (!empty($nick)) {
				$title = $nick . '(' . $userid . ')';
			} else {
				$title = $userid;
			}
		}

		// Set it in cache
		$GLOBALS[__FUNCTION__][$userid][$title . '_' . $what] = '[<a href="{%url=modules.php?module=admin&amp;what=' . $what . '&amp;userid=' . $userid . '%}" title="{--ADMIN_USER_PROFILE_TITLE--}">' . $title . '</a>]';
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$userid][$title . '_' . $what];
}

// Check "logical-area-mode"
function adminGetMenuMode () {
	// Set the default menu mode as the mode for all admins
	$mode = 'global';

	// If ext-sql_patches is up-to-date enough, use the configuration
	if (isExtensionInstalledAndNewer('sql_patches', '0.3.2')) {
		$mode = getAdminMenu();
	} // END - if

	// Backup it
	$adminMode = $mode;

	// Get admin id
	$adminId = getCurrentAdminId();

	// Check individual settings of current admin
	if (isset($GLOBALS['cache_array']['admin']['la_mode'][$adminId])) {
		// Load from cache
		$adminMode = $GLOBALS['cache_array']['admin']['la_mode'][$adminId];
		incrementStatsEntry('cache_hits');
	} elseif (isExtensionInstalledAndNewer('admins', '0.6.7')) {
		// Load from database when version of 'admins' is enough
		$result = SQL_QUERY_ESC("SELECT `la_mode` FROM `{?_MYSQL_PREFIX?}_admins` WHERE `id`=%s LIMIT 1",
			array($adminId), __FUNCTION__, __LINE__);

		// Is there an entry?
		if (SQL_NUMROWS($result) == 1) {
			// Load data
			list($adminMode) = SQL_FETCHROW($result);
		} // END - if

		// Free memory
		SQL_FREERESULT($result);
	}

	// Check what the admin wants and set it when it's not the default mode
	if ($adminMode != 'global') {
		$mode = $adminMode;
	} // END - if

	// Return admin-menu's mode
	return $mode;
}

// Change activation status
function adminChangeActivationStatus ($IDs, $table, $row, $idRow = 'id') {
	$count = '0';
	if ((is_array($IDs)) && (count($IDs) > 0)) {
		// "Walk" all through and count them
		foreach ($IDs as $id => $selected) {
			// Secure the id number
			$id = bigintval($id);

			// Should always be set... ;-)
			if (!empty($selected)) {
				// Determine new status
				$result = SQL_QUERY_ESC("SELECT %s FROM `{?_MYSQL_PREFIX?}_%s` WHERE %s=%s LIMIT 1",
					array(
						$row,
						$table,
						$idRow,
						$id
					), __FUNCTION__, __LINE__);

				// Row found?
				if (SQL_NUMROWS($result) == 1) {
					// Load the status
					list($currStatus) = SQL_FETCHROW($result);

					// And switch it N<->Y
					$newStatus = convertBooleanToYesNo(!($currStatus == 'Y'));

					// Change this status
					SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_%s` SET %s='%s' WHERE %s=%s LIMIT 1",
						array(
							$table,
							$row,
							$newStatus,
							$idRow,
							$id
						), __FUNCTION__, __LINE__);

					// Count up affected rows
					$count += SQL_AFFECTEDROWS();
				} // END - if

				// Free the result
				SQL_FREERESULT($result);
			} // END - if
		} // END - foreach

		// Output status
		displayMessage(sprintf(getMessage('ADMIN_STATUS_CHANGED'), $count, count($IDs)));
	} else {
		// Nothing selected!
		displayMessage('{--ADMIN_NOTHING_SELECTED_CHANGE--}');
	}
}

// Build a special template list
// @TODO cacheFiles is not yet supported
function adminListBuilder ($listType, $tableName, $columns, $filterFunctions, $extraValues, $idColumn, $userIdColumn, $rawUserId = array('userid'), $content = array()) {
	// Call inner (general) function
	doGenericListBuilder('admin', $listType, $tableName, $columns, $filterFunctions, $extraValues, $idColumn, $userIdColumn, $rawUserId, $content);
}

// Change status of "build" list
function adminBuilderStatusHandler ($mode, $tableName, $columns, $filterFunctions, $extraValues, $idColumn, $userIdColumn, $statusArray, $rawUserId = array('userid'), $cacheFiles = array()) {
	// $tableName must be an array
	if ((!is_array($tableName)) || (count($tableName) != 1)) {
		// No tableName specified
		reportBug(__FUNCTION__, __LINE__, 'tableName is not given. Please fix your XML,tableName[]=' . gettype($tableName) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($idColumn)) {
		// $idColumn is no array
		reportBug(__FUNCTION__, __LINE__, 'idColumn[]=' . gettype($idColumn) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif ((!is_array($userIdColumn)) || (count($userIdColumn) != 1)) {
		// $tableName is no array
		reportBug(__FUNCTION__, __LINE__, 'userIdColumn[]=' . gettype($userIdColumn) . '!=array: userIdColumn=' . $userIdColumn);
	} // END - if

	// "Walk" through all entries
	foreach (postRequestElement($idColumn[0]) as $id => $sel) {
		// Construct SQL query
		$sql = sprintf("UPDATE `{?_MYSQL_PREFIX?}_%s` SET", SQL_ESCAPE($tableName[0]));

		// Load data of entry
		$result = SQL_QUERY_ESC("SELECT * FROM `{?_MYSQL_PREFIX?}_%s` WHERE `%s`=%s LIMIT 1",
			array(
				$tableName[0],
				$idColumn[0],
				$id
			), __FUNCTION__, __LINE__);

		// Fetch the data
		$content = SQL_FETCHARRAY($result);

		// Free the result
		SQL_FREERESULT($result);

		// Add all status entries (e.g. status column last_updated or so)
		$newStatus = 'UNKNOWN';
		$oldStatus = 'UNKNOWN';
		$statusColumn = 'unknown';
		foreach ($statusArray as $column => $statusInfo) {
			// Does the entry exist?
			if ((isset($content[$column])) && (isset($statusInfo[$content[$column]]))) {
				// Add these entries for update
				$sql .= sprintf(" `%s`='%s',", SQL_ESCAPE($column), SQL_ESCAPE($statusInfo[$content[$column]]));

				// Remember status
				if ($statusColumn == 'unknown') {
					// Always (!!!) change status column first!
					$oldStatus = $content[$column];
					$newStatus = $statusInfo[$oldStatus];
					$statusColumn = $column;
				} // END - if
			} elseif (isset($content[$column])) {
				// Unfinished!
				reportBug(__FUNCTION__, __LINE__, ':UNFINISHED: id=' . $id . ',column=' . $column . '[' . gettype($statusInfo) . '] = ' . $content[$column]);
			}
		} // END - foreach

		// Add other columns as well
		foreach (postRequestArray() as $key => $entries) {
			// Debug message
			/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Found entry: ' . $key);

			// Skip id, raw userid and 'do_$mode'
			if (!in_array($key, array($idColumn[0], $rawUserId[0], ('do_' . $mode)))) {
				// Are there brackets () at the end?
				if (substr($entries[$id], -2, 2) == '()') {
					// Direct SQL command found
					$sql .= sprintf(" `%s`=%s,", SQL_ESCAPE($key), SQL_ESCAPE($entries[$id]));
				} else {
					// Add regular entry
					$sql .= sprintf(" `%s`='%s',", SQL_ESCAPE($key), SQL_ESCAPE($entries[$id]));

					// Add entry
					$content[$key] = $entries[$id];
				}
			} elseif (isDebugModeEnabled()) {
				// Skipped entry
				logDebugMessage(__FUNCTION__, __LINE__, 'Skipped: ' . $key);
			}
		} // END - foreach

		// Finish SQL statement
		$sql = substr($sql, 0, -1) . sprintf(" WHERE `%s`=%s AND `%s`='%s' LIMIT 1",
			$idColumn[0],
			bigintval($id),
			$statusColumn,
			$oldStatus
		);

		// Run the SQL
		SQL_QUERY($sql, __FUNCTION__, __LINE__);

		// Send "build mails" out
		sendGenericBuildMails($mode, $tableName, $content, $id, $statusInfo[$content[$column]], $userIdColumn);
	} // END - foreach
}

// Delete rows by given id numbers
function adminDeleteEntriesConfirm ($tableName, $columns = array(), $filterFunctions = array(), $extraValues = array(), $deleteNow = array(FALSE), $idColumn = array('id'), $userIdColumn = array('userid'), $rawUserId = array('userid'), $cacheFiles = array(), $content = array()) {
	// $tableName must be an array
	if ((!is_array($tableName)) || (count($tableName) != 1)) {
		// No tableName specified
		reportBug(__FUNCTION__, __LINE__, 'tableName is not given. Please fix your XML,tableName[]=' . gettype($tableName) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($idColumn)) {
		// $idColumn is no array
		reportBug(__FUNCTION__, __LINE__, 'idColumn[]=' . gettype($idColumn) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($userIdColumn)) {
		// $userIdColumn is no array
		reportBug(__FUNCTION__, __LINE__, 'userIdColumn[]=' . gettype($userIdColumn) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($deleteNow)) {
		// $deleteNow is no array
		reportBug(__FUNCTION__, __LINE__, 'deleteNow[]=' . gettype($deleteNow) . '!=array: userIdColumn=' . $userIdColumn);
	} // END - if

	// Shall we delete here or list for deletion?
	if ($deleteNow[0] === TRUE) {
		// Call generic function
		$affected = doGenericDeleteEntriesConfirm($tableName, $columns, $filterFunctions, $extraValues, $deleteNow, $idColumn, $userIdColumn, $rawUserId, $cacheFiles);

		// Was this fine?
		if ($affected == countPostSelection($idColumn[0])) {
			// All deleted
			displayMessage('{--ADMIN_ALL_ENTRIES_REMOVED--}');
		} else {
			// Some are still there :(
			displayMessage(sprintf(getMessage('ADMIN_SOME_ENTRIES_NOT_DELETED'), SQL_AFFECTEDROWS(), countPostSelection($idColumn[0])));
		}
	} else {
		// List for deletion confirmation
		adminListBuilder('delete', $tableName, $columns, $filterFunctions, $extraValues, $idColumn, $userIdColumn, $rawUserId, $content);
	}
}

// Edit rows by given id numbers
function adminEditEntriesConfirm ($tableName, $columns = array(), $filterFunctions = array(), $extraValues = array(), $timeColumns = array(), $editNow = array(FALSE), $idColumn = array('id'), $userIdColumn = array('userid'), $rawUserId = array('userid'), $cacheFiles = array(), $content = array()) {
	// $tableName must be an array
	if ((!is_array($tableName)) || (count($tableName) != 1)) {
		// No tableName specified
		reportBug(__FUNCTION__, __LINE__, 'tableName is not given. Please fix your XML,tableName[]=' . gettype($tableName) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($idColumn)) {
		// $idColumn is no array
		reportBug(__FUNCTION__, __LINE__, 'idColumn[]=' . gettype($idColumn) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($userIdColumn)) {
		// $userIdColumn is no array
		reportBug(__FUNCTION__, __LINE__, 'userIdColumn[]=' . gettype($userIdColumn) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($editNow)) {
		// $editNow is no array
		reportBug(__FUNCTION__, __LINE__, 'editNow[]=' . gettype($editNow) . '!=array: userIdColumn=' . $userIdColumn);
	} // END - if

	// Shall we change here or list for editing?
	if ($editNow[0] === TRUE) {
		// Call generic change method
		$affected = doGenericEditEntriesConfirm($tableName, $columns, $filterFunctions, $extraValues, $timeColumns, $editNow, $idColumn, $userIdColumn, $rawUserId, $cacheFiles);

		// Was this fine?
		if ($affected == countPostSelection($idColumn[0])) {
			// All deleted
			displayMessage('{--ADMIN_ALL_ENTRIES_EDITED--}');
		} else {
			// Some are still there :(
			displayMessage(sprintf(getMessage('ADMIN_SOME_ENTRIES_NOT_EDITED'), $affected, countPostSelection($idColumn[0])));
		}
	} else {
		// List for editing
		adminListBuilder('edit', $tableName, $columns, $filterFunctions, $extraValues, $idColumn, $userIdColumn, $rawUserId, $content);
	}
}

// Un-/lock rows by given id numbers
// @TODO rawUserId/content is not yet supported
function adminLockEntriesConfirm ($tableName, $columns = array(), $filterFunctions = array(), $extraValues = array(), $statusArray = array(), $lockNow = array(FALSE), $idColumn = array('id'), $userIdColumn = array('userid')) {
	// $tableName must be an array
	if ((!is_array($tableName)) || (count($tableName) != 1)) {
		// No tableName specified
		reportBug(__FUNCTION__, __LINE__, 'tableName is not given. Please fix your XML,tableName[]=' . gettype($tableName) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($idColumn)) {
		// $idColumn is no array
		reportBug(__FUNCTION__, __LINE__, 'idColumn[]=' . gettype($idColumn) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($lockNow)) {
		// $lockNow is no array
		reportBug(__FUNCTION__, __LINE__, 'lockNow[]=' . gettype($lockNow) . '!=array: userIdColumn=' . $userIdColumn);
	} // END - if

	// Shall we un-/lock here or list for locking?
	if ($lockNow[0] === TRUE) {
		// Un-/lock entries
		adminBuilderStatusHandler('lock', $tableName, $columns, $filterFunctions, $extraValues, $idColumn, $userIdColumn, $statusArray);
	} else {
		// List for editing
		adminListBuilder('lock', $tableName, $columns, $filterFunctions, $extraValues, $idColumn, $userIdColumn);
	}
}

// Undelete rows by given id numbers
// @TODO rawUserId/cacheFiles/content is not yet supported
function adminUndeleteEntriesConfirm ($tableName, $columns = array(), $filterFunctions = array(), $extraValues = array(), $statusArray = array(), $undeleteNow = array(FALSE), $idColumn = array('id'), $userIdColumn = array('userid')) {
	// $tableName must be an array
	if ((!is_array($tableName)) || (count($tableName) != 1)) {
		// No tableName specified
		reportBug(__FUNCTION__, __LINE__, 'tableName is not given. Please fix your XML,tableName[]=' . gettype($tableName) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($idColumn)) {
		// $idColumn is no array
		reportBug(__FUNCTION__, __LINE__, 'idColumn[]=' . gettype($idColumn) . '!=array: userIdColumn=' . $userIdColumn);
	} elseif (!is_array($undeleteNow)) {
		// $undeleteNow is no array
		reportBug(__FUNCTION__, __LINE__, 'undeleteNow[]=' . gettype($undeleteNow) . '!=array: userIdColumn=' . $userIdColumn);
	} // END - if

	// Shall we un-/lock here or list for locking?
	if ($undeleteNow[0] === TRUE) {
		// Undelete entries
		adminBuilderStatusHandler('undelete', $tableName, $columns, $filterFunctions, $extraValues, $idColumn, $userIdColumn, $statusArray);
	} else {
		// List for editing
		adminListBuilder('undelete', $tableName, $columns, $filterFunctions, $extraValues, $idColumn, $userIdColumn);
	}
}

// Adds a given entry to the database
function adminAddEntries ($tableName, $columns = array(), $filterFunctions = array(), $extraValues = array(), $timeColumns = array(), $columnIndex = NULL) {
	// Is the userid set?
	if (!isPostRequestElementSet('userid')) {
		// Then set NULL here
		setPostRequestElement('userid', NULL);
	} // END - if

	// Call inner function
	doGenericAddEntries($tableName, $columns, $filterFunctions, $extraValues, $timeColumns, $columnIndex);

	// Entry has been added?
	if ((!SQL_HASZEROAFFECTED()) && ($GLOBALS['__XML_PARSE_RESULT'] === TRUE)) {
		// Display success message
		displayMessage('{--ADMIN_ENTRY_ADDED--}');
	} else {
		// Display failed message
		displayMessage('{--ADMIN_ENTRY_NOT_ADDED--}');
	}
}

// Checks proxy settins by fetching check-updates3.php from mxchange.org
function adminTestProxySettings ($settingsArray) {
	// Set temporary the new settings
	mergeConfig($settingsArray);

	// Now get the test URL
	$content = sendHttpGetRequest('check-updates3.php');

	// Is the first line with "200 OK"?
	$valid = isInString('200 OK', $content[0]);

	// Return result
	return $valid;
}

// Sends out a link to the given email adress so the admin can reset his/her password
function sendAdminPasswordResetLink ($email) {
	// Init output
	$OUT = '';

	// Look up administator login
	$result = SQL_QUERY_ESC("SELECT `id`, `login`, `password` FROM `{?_MYSQL_PREFIX?}_admins` WHERE '%s' REGEXP `email` LIMIT 1",
		array($email), __FUNCTION__, __LINE__);

	// Is there an account?
	if (SQL_HASZERONUMS($result)) {
		// No account found
		return '{--ADMIN_NO_LOGIN_WITH_EMAIL--}';
	} // END - if

	// Load all data
	$content = SQL_FETCHARRAY($result);

	// Free result
	SQL_FREERESULT($result);

	// Generate hash for reset link
	$content['hash'] = generateHash(getUrl() . getEncryptSeparator() . $content['id'] . getEncryptSeparator() . $content['login'] . getEncryptSeparator() . $content['password'], substr($content['password'], getSaltLength()));

	// Remove some data
	unset($content['id']);
	unset($content['password']);

	// Prepare email
	$mailText = loadEmailTemplate('admin_reset_password', $content);

	// Send it out
	sendEmail($email, '{--ADMIN_RESET_PASSWORD_LINK_SUBJECT--}', $mailText);

	// Prepare output
	return '{--ADMIN_RESET_PASSWORD_LINK_SENT--}';
}

// Validate hash and login for password reset
function adminResetValidateHashLogin ($hash, $login) {
	// By default nothing validates... ;)
	$valid = FALSE;

	// Then try to find that user
	$result = SQL_QUERY_ESC("SELECT `id`, `password`, `email` FROM `{?_MYSQL_PREFIX?}_admins` WHERE `login`='%s' LIMIT 1",
		array($login), __FUNCTION__, __LINE__);

	// Is an account here?
	if (SQL_NUMROWS($result) == 1) {
		// Load all data
		$content = SQL_FETCHARRAY($result);

		// Generate hash again
		$hashFromData = generateHash(getUrl() . getEncryptSeparator() . $content['id'] . getEncryptSeparator() . $login . getEncryptSeparator() . $content['password'], substr($content['password'], getSaltLength()));

		// Does both match?
		$valid = ($hash == $hashFromData);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $valid;
}

// Reset the password for the login. Do NOT call this function without calling above function first!
function doResetAdminPassword ($login, $password) {
	// Generate hash (we already check for ext-sql_patches in generateHash())
	$passHash = generateHash($password);

	// Prepare fake POST data
	$postData = array(
		'login'    => array(getAdminId($login) => $login),
		'password' => array(getAdminId($login) => $passHash),
	);

	// Update database
	$message = adminsChangeAdminAccount($postData, '', FALSE);

	// Run filters
	runFilterChain('post_form_reset_pass', array('login' => $login, 'hash' => $passHash, 'message' => $message));

	// Return output
	return '{--ADMIN_PASSWORD_RESET_DONE--}';
}

// Solves a task by given id number
function adminSolveTask ($id) {
	// Update the task data
	adminUpdateTaskData($id, 'status', 'SOLVED');
}

// Marks a given task as deleted
function adminDeleteTask ($id) {
	// Update the task data
	adminUpdateTaskData($id, 'status', 'DELETED');
}

// Function to update task data
function adminUpdateTaskData ($id, $row, $data) {
	// Should be admin and valid id
	if (!isAdmin()) {
		// Not an admin so redirect better
		reportBug(__FUNCTION__, __LINE__, 'id=' . $id . ',row=' . $row . ',data=' . $data . ' - isAdmin()=false');
	} elseif ($id <= 0) {
		// Initiate backtrace
		reportBug(__FUNCTION__, __LINE__, sprintf("id is invalid: %s. row=%s, data=%s",
			$id,
			$row,
			$data
		));
	} // END - if

	// Update the task
	SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_task_system` SET `%s`='%s' WHERE `id`=%s LIMIT 1",
		array(
			$row,
			$data,
			bigintval($id)
		), __FUNCTION__, __LINE__);
}

// Checks whether if the admin menu has entries
function ifAdminMenuHasEntries ($action) {
	return (
		((
			// Is the entry set?
			isset($GLOBALS['admin_menu_has_entries'][$action])
		) && (
			// And do we have a menu for this action?
			$GLOBALS['admin_menu_has_entries'][$action] === TRUE
		)) || (
			// Login has always a menu
			$action == 'login'
		)
	);
}

// Setter for 'admin_menu_has_entries'
function setAdminMenuHasEntries ($action, $hasEntries) {
	$GLOBALS['admin_menu_has_entries'][$action] = (bool) $hasEntries;
}

// Creates a link to the user's admin-profile
function adminCreateUserLink ($userid) {
	// Is the userid set correctly?
	if (isValidId($userid)) {
		// Create a link to that profile
		return '{%url=modules.php?module=admin&amp;what=list_user&amp;userid=' . bigintval($userid) . '%}';
	} // END - if

	// Return a link to the user list
	return '{%url=modules.php?module=admin&amp;what=list_user%}';
}

// Generate a "link" for the given admin id (admin_id)
function generateAdminLink ($adminId) {
	// No assigned admin is default
	$adminLink = '{--ADMIN_NO_ADMIN_ASSIGNED--}';

	// Zero? = Not assigned
	if (isValidId($adminId)) {
		// Load admin's login
		$login = getAdminLogin($adminId);

		// Is the login valid?
		if ($login != '***') {
			// Is the extension there?
			if (isExtensionActive('admins')) {
				// Admin found
				$adminLink = '<a href="' . generateEmailLink(getAdminEmail($adminId), 'admins') . '" title="{--ADMIN_CONTACT_LINK_TITLE--}">' . $login . '</a>';
			} else {
				// Extension not found
				$adminLink = '{%message,ADMIN_TASK_ROW_EXTENSION_NOT_INSTALLED=admins%}';
			}
		} else {
			// Maybe deleted?
			$adminLink = '<div class="bad">{%message,ADMIN_ID_404=' . $adminId . '%}</div>';
		}
	} // END - if

	// Return result
	return $adminLink;
}

// Verifies if the current admin has confirmed to alter expert settings
//
// Return values:
// 'failed'    = Something goes wrong (default)
// 'agreed'    = Has verified and and confirmed it to see them
// 'forbidden' = Has not the proper right to alter them
// 'update'    = Need to update extension 'admins'
// 'ask'       = A form was send to the admin
function doVerifyExpertSettings () {
	// Default return status is failed
	$return = 'failed';

	// Is the extension installed and recent?
	if (isExtensionInstalledAndNewer('admins', '0.7.3')) {
		// Okay, load the status
		$expertSettings = getAminsExpertSettings();

		// Is he allowed?
		if ($expertSettings == 'Y') {
			// Okay, does he want to see them?
			if (isAdminsExpertWarningEnabled()) {
				// Ask for them
				if (isFormSent('save_expert')) {
					// Is the element set, then we need to change the admin
					if (isPostRequestElementSet('expert_settings')) {
						// Get it and prepare final post data array
						$postData['login'][getCurrentAdminId()] = getCurrentAdminLogin();
						$postData['expert_warning'][getCurrentAdminId()] = 'N';

						// Change it in the admin
						adminsChangeAdminAccount($postData, 'expert_warning');
					} // END - if

					// All fine!
					$return = 'agreed';
				} else {
					// Send form
					loadTemplate('admin_expert_settings_form');

					// Asked for it
					$return = 'ask';
				}
			} else {
				// Do not display
				$return = 'agreed';
			}

			// Is a form sent?
			if ((isFormSent()) && (isPostRequestElementSet('expert_settings'))) {
				// Clear form
				unsetPostRequestElement('ok');
				unsetPostRequestElement('expert_settings');
			} // END - if
		} else {
			// Forbidden
			$return = 'forbidden';
		}
	} else {
		// Out-dated extension or not installed
		$return = 'update';
	}

	// Output message for other status than ask/agreed
	if (($return != 'ask') && ($return != 'agreed')) {
		// Output message
		displayMessage('{--ADMIN_EXPERT_SETTINGS_STATUS_' . strtoupper($return) . '--}');
	} // END - if

	// Return status
	return $return;
}

// Generate link to unconfirmed mails for admin
function generateUnconfirmedAdminLink ($id, $unconfirmed, $type) {
	// Init output
	$OUT = $unconfirmed;

	// Is there unconfirmed mails?
	if ($unconfirmed > 0) {
		// Add link to list_unconfirmed what-file
		$OUT = '<a href="{%url=modules.php?module=admin&amp;what=list_unconfirmed&amp;type=' . $type . '&amp;id=' . $id . '%}">{%pipe,translateComma=' . $unconfirmed . '%}</a>';
	} // END - if

	// Return it
	return $OUT;
}

// Generates a navigation row for listing emails
function addEmailNavigation ($numPages, $offset, $show_form, $colspan, $return=false) {
	// Don't do anything if $numPages is 1
	if ($numPages == 1) {
		// Abort here with empty content
		return '';
	} // END - if

	$TOP = '';
	if ($show_form === FALSE) {
		$TOP = ' top';
	} // END - if

	$NAV = '';
	for ($page = 1; $page <= $numPages; $page++) {
		// Is the page currently selected or shall we generate a link to it?
		if (($page == getRequestElement('page')) || ((!isGetRequestElementSet('page')) && ($page == 1))) {
			// Is currently selected, so only highlight it
			$NAV .= '<strong>-';
		} else {
			// Open anchor tag and add base URL
			$NAV .= '<a href="{%url=modules.php?module=admin&amp;what=' . getWhat() . '&amp;page=' . $page . '&amp;offset=' . $offset;

			// Add userid when we shall show all mails from a single member
			if ((isGetRequestElementSet('userid')) && (isValidId(getRequestElement('userid')))) $NAV .= '&amp;userid=' . bigintval(getRequestElement('userid'));

			// Close open anchor tag
			$NAV .= '%}">';
		}
		$NAV .= $page;
		if (($page == getRequestElement('page')) || ((!isGetRequestElementSet('page')) && ($page == 1))) {
			// Is currently selected, so only highlight it
			$NAV .= '-</strong>';
		} else {
			// Close anchor tag
			$NAV .= '</a>';
		}

		// Add separator if total pages has not been reached
		if ($page < $numPages) {
			// Add it
			$NAV .= '|';
		} // END - if
	} // END - for

	// Define constants only once
	$content['nav']  = $NAV;
	$content['span'] = $colspan;
	$content['top']  = $TOP;

	// Load navigation template
	$OUT = loadTemplate('admin_email_nav_row', TRUE, $content);

	if ($return === TRUE) {
		// Return generated HTML-Code
		return $OUT;
	} else {
		// Output HTML-Code
		outputHtml($OUT);
	}
}

// Process menu editing form
function adminProcessMenuEditForm ($type, $subMenu) {
	// An action is done...
	foreach (postRequestElement('sel') as $sel => $menu) {
		$AND = "(`what` = '' OR `what` IS NULL)";

		$sel = bigintval($sel);

		if (!empty($subMenu)) {
			$AND = "`action`='" . $subMenu . "'";
		} // END - if

		switch (postRequestElement('ok')) {
			case 'edit': // Edit menu
				// Shall we update a menu or sub menu?
				if (!isGetRequestElementSet('sub')) {
					// Update with 'what'=null
					SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_%s_menu` SET `title`='%s',`action`='%s',`what`=NULL WHERE ".$AND." AND `id`=%s LIMIT 1",
						array(
							$type,
							$menu,
							postRequestElement('sel_action', $sel),
							$sel
						), __FUNCTION__, __LINE__);
				} else {
					// Update with selected 'what'
					SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_%s_menu` SET `title`='%s',`action`='%s',`what`='%s' WHERE ".$AND." AND `id`=%s LIMIT 1",
						array(
							$type,
							$menu,
							postRequestElement('sel_action', $sel),
							postRequestElement('sel_what', $sel),
							$sel
						), __FUNCTION__, __LINE__);
				}
				break;

			case 'delete': // Delete menu
				SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_%s_menu` WHERE ".$AND." AND `id`=%s LIMIT 1",
					array(
						$type,
						$sel
					), __FUNCTION__, __LINE__);
				break;

			case 'status': // Change status of menus
				SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_%s_menu` SET `visible`='%s',`locked`='%s' WHERE ".$AND." AND `id`=%s LIMIT 1",
					array(
						$type,
						postRequestElement('visible', $sel),
						postRequestElement('locked', $sel),
						$sel
					), __FUNCTION__, __LINE__);
				break;

			default: // Unexpected action
				logDebugMessage(__FUNCTION__, __LINE__, sprintf("Unsupported action %s detected.", postRequestElement('ok')));
				displayMessage('{%message,ADMIN_UNKNOWN_OKAY=' . postRequestElement('ok') . '%}');
				break;
		} // END - switch
	} // END - foreach

	// Load template
	displayMessage('{--SETTINGS_SAVED--}');
}

// Handle weightning
function doAdminProcessMenuWeightning ($type, $AND) {
	// Are there all required (generalized) GET parameter?
	if ((isGetRequestElementSet('act')) && (isGetRequestElementSet('tid')) && (isGetRequestElementSet('fid'))) {
		// Init variables
		$tid = ''; $fid = '';

		// Get ids
		if (isGetRequestElementSet('w')) {
			// Sub menus selected
			$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_%s_menu` WHERE `action`='%s' AND `sort`=%s LIMIT 1",
				array(
					$type,
					getRequestElement('act'),
					bigintval(getRequestElement('tid'))
				), __FUNCTION__, __LINE__);
			list($tid) = SQL_FETCHROW($result);
			SQL_FREERESULT($result);
			$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_%s_menu` WHERE `action`='%s' AND `sort`=%s LIMIT 1",
				array(
					$type,
					getRequestElement('act'),
					bigintval(getRequestElement('fid'))
				), __FUNCTION__, __LINE__);
			list($fid) = SQL_FETCHROW($result);
			SQL_FREERESULT($result);
		} else {
			// Main menu selected
			$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_%s_menu` WHERE (`what`='' OR `what` IS NULL) AND `sort`=%s LIMIT 1",
				array(
					$type,
					bigintval(getRequestElement('tid'))
				), __FUNCTION__, __LINE__);
			list($tid) = SQL_FETCHROW($result);
			SQL_FREERESULT($result);
			$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_%s_menu` WHERE (`what`='' OR `what` IS NULL) AND `sort`=%s LIMIT 1",
				array(
					$type,
					bigintval(getRequestElement('fid'))
				), __FUNCTION__, __LINE__);
			list($fid) = SQL_FETCHROW($result);
			SQL_FREERESULT($result);
		}

		if ((!empty($tid)) && (!empty($fid))) {
			// Sort menu
			SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_%s_menu` SET `sort`=%s WHERE ".$AND." AND `id`=%s LIMIT 1",
				array(
					$type,
					bigintval(getRequestElement('tid')),
					bigintval($fid)
				), __FUNCTION__, __LINE__);
			SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_%s_menu` SET `sort`=%s WHERE ".$AND." AND `id`=%s LIMIT 1",
				array(
					$type,
					bigintval(getRequestElement('fid')),
					bigintval($tid)
				), __FUNCTION__, __LINE__);
		} // END - if
	} // END - if
}

// Function to register first admin
function registerFirstAdmin () {
	// Make sure that no admin is registered
	assert(!isAdminRegistered());

	// Admin is not registered so we have to inform the user
	if ((isFormSent('add_first_admin')) && ((!isPostRequestElementSet('admin_login')) || (!isPostRequestElementSet('admin_password1')) || (strlen(postRequestElement('admin_password1')) < getConfig('minium_admin_pass_length')) || (!isPostRequestElementSet('admin_password2')) || (strlen(postRequestElement('admin_password2')) < getConfig('minium_admin_pass_length')) || (postRequestElement('admin_password1') != postRequestElement('admin_password2')))) {
		setPostRequestElement('add_first_admin', '***');
	} // END - if

	// Clear error message
	$errorMessage = '';
	$ret = 'init';

	// Is form for first admin sent?
	if ((isFormSent('add_first_admin')) && (postRequestElement('add_first_admin') != '***')) {
		// Hash the password with the old function because we are here in install mode
		$hashedPass = md5(postRequestElement('admin_password1'));

		// Kill maybe existing session variables
		destroyAdminSession();

		// Do registration
		$ret = addAdminAccount(postRequestElement('admin_login'), $hashedPass, getWebmaster(), 'allow');

		// Check if registration wents fine
		switch ($ret) {
			case 'done':
				// Change ADMIN_REGISTERED entry
				$done = changeDataInLocalConfigurationFile('ADMIN-SETUP', "setConfigEntry('ADMIN_REGISTERED', '", "');", 'Y', 0);

				// Was it successfull?
				if ($done === TRUE) {
					// Registering is done
					redirectToUrl('modules.php?module=admin&amp;register=done');
				} else {
					// Registration incomplete
					$errorMessage = '{--ADMIN_CANNOT_COMPLETE--}';

					// Set this to have our error message displayed
					setPostRequestElement('add_first_admin', '***');
				}
				break;

			case 'failed': // Registration has failed
				$errorMessage = '{--ADMIN_REGISTER_FAILED--}';

				// Set this to have our error message displayed
				setPostRequestElement('add_first_admin', '***');
				break;

			case 'already': // Admin does already exists!
				$errorMessage = '{--ADMIN_LOGIN_ALREADY_REG--}';

				// Set this to have our error message displayed
				setPostRequestElement('add_first_admin', '***');
				break;

			default:
				// Any other kind will be logged
				$errorMessage = sprintf("Unknown return code %s from ifAdminLoginDataIsValid().", $ret);
				logDebugMessage(__FUNCTION__, __LINE__, $errorMessage);

				// Set this to have our error message displayed
				setPostRequestElement('add_first_admin', '***');
				break;
		} // END - switch
	} // END - if

	// Whas that action okay?
	if ($ret != 'done') {
		// Init login name
		$content['admin_login'] = '';
		if (isPostRequestElementSet('admin_login')) {
			$content['admin_login'] = postRequestElement('admin_login');
		} // END - if

		// Init array elements
		$content['login_message'] = '';
		$content['password1_message'] = '';
		$content['password2_message'] = '';
		$content['error_message'] = '';

		// Yet-another notice-fix
		if ((isFormSent('add_first_admin')) && (postRequestElement('add_first_admin') == '***')) {
			// Init variables
			$loginMessage = '';
			$password1Message = '';
			$password2Message = '';

			// No login entered?
			if (empty($content['admin_login'])) {
				$loginMessage = '{--ADMIN_NO_LOGIN--}';
			} // END - if

			// An error comes back from registration?
			if ((!empty($ret)) && ($ret != 'init')) {
				$loginMessage = $errorMessage;
			} // END - if

			// No password 1 entered or to short?
			if (!isPostRequestElementSet('admin_password1')) {
				$password1Message = '{--ADMIN_NO_PASSWORD1--}';
			} elseif (strlen(postRequestElement('admin_password1')) < getConfig('minium_admin_pass_length')) {
				$password1Message = '{--ADMIN_SHORT_PASSWORD1--}';
			}

			// No password 2 entered or to short?
			if (!isPostRequestElementSet('admin_password2')) {
				$password2Message = '{--ADMIN_NO_PASSWORD2--}';
			} elseif (strlen(postRequestElement('admin_password2')) < getConfig('minium_admin_pass_length')) {
				$password2Message = '{--ADMIN_SHORT_PASSWORD2--}';
			}

			// Both didn't match?
			if (postRequestElement('admin_password1') != postRequestElement('admin_password2')) {
				// No match
				if (empty($password1Message)) $password1Message = '{--ADMIN_PASSWORD1_MISMATCH--}';
				if (empty($password2Message)) $password2Message = '{--ADMIN_PASSWORD2_MISMATCH--}';
			} // END - if

			// Output error messages
			$content['login_message']     = loadTemplate('admin_login_msg', TRUE, $loginMessage);
			$content['password1_message'] = loadTemplate('admin_login_msg', TRUE, $password1Message);
			$content['password2_message'] = loadTemplate('admin_login_msg', TRUE, $password2Message);
			$content['error_message']     = loadTemplate('admin_login_msg', TRUE, $errorMessage);
		} // END - if

		// Output message in seperate template
		displayMessage('{--ADMIN_ACCOUNT_NOT_REGISTERED_YET--}');

		// Load register template
		loadTemplate('admin_reg_form', FALSE, $content);
	} // END - if
}

// [EOF]
?>
